<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xalan"
                xmlns:r="http://software.in2p3.fr/lavoisier/tables.xsd"
                exclude-result-prefixes="xalan">
    <!-- ******************************************* -->
    <!-- Centre de Calcul de l'IN2P3 - Lyon (France) -->
    <!--             http://cc.in2p3.fr/             -->
    <!-- ******************************************* -->
    <!-- author: Jonathan Moutarde                   -->
    <!-- ******************************************* -->
    <xsl:output method="xml" indent="yes" xalan:indent-amount="4"/>

    <xsl:variable name="parameters" select="document('options')/r:options"/>

    <xsl:variable name="stacking" select="$parameters/@stacking"/>
    <xsl:variable name="axis" select="$parameters/@axis"/>
    <xsl:variable name="unit" select="$parameters/r:unit/r:entry"/>
    <xsl:variable name="stacked" select="$parameters/r:stacked/r:entry"/>
    <xsl:variable name="types" select="$parameters/r:types/r:entry"/>
    <xsl:variable name="type" select="$types[@key='*']"/>
    <xsl:variable name="options" select="$parameters/r:sections/r:section"/>

    <xsl:variable name="xAxis" select="$parameters/r:xAxis" />
    <xsl:variable name="yAxis" select="$parameters/r:yAxis|$parameters/r:yAxis2|$parameters/r:yAxis3" />

    <xsl:variable name="xAxisMode">
        <xsl:choose>
            <xsl:when test="$xAxis/r:column">
                <xsl:value-of select="'xAxis'" />
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="'unit'" />
            </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    <xsl:variable name="yAxisMode">
        <xsl:choose>
            <xsl:when test="$yAxis[name(.)='yAxis']/r:column">
                <xsl:value-of select="'yAxis'" />
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="'unit'" />
            </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>

    <xsl:variable name="highcharts-colors"
                    xmlns:str="http://exslt.org/strings"
                    select="str:split('4572A7,AA4643,89A54E,80699B,3D96AE,DB843D,92A8CD,A47D7C,B5CA92,4572A7', ',')" />


    <!-- prevents unwanted text() nodes to be printed -->
    <xsl:template match="/r:tables">
        <xsl:if test="(($xAxisMode='unit') and ($yAxisMode='yAxis')) or (($xAxisMode='xAxis') and ($yAxisMode='unit'))">
            <xsl:message terminate="yes">Hybrid mode not supported. Specify both xAxis and yAxis parameters for renderer or none of them.
xAxis : <xsl:copy-of select="$xAxis" />
yAxis : <xsl:copy-of select="$yAxis" /></xsl:message>
        </xsl:if>
        <xsl:if test="$parameters and
            (($options[@key='chart']/r:section[(@key='type') or (@key='inverted') or (@key='polar')]) or
            ($options[@key='title']/r:section[(@key='text') or (@key='style')]) or
            ($options[@key='plotOption']//r:section[@key='stacking']))">
            <xsl:message terminate="yes">A marche pas.</xsl:message>
        </xsl:if>

        <supermap>
            <xsl:for-each select="./r:table">
                <map>
                    <!-- TODO: Penser à faire les vérification avant de créer les attributs au cas ou un template aurait un retour vide. -->
                    <xsl:apply-templates select="." mode="CHART"/>
                    <xsl:apply-templates select="." mode="AXIS"/>
                    <xsl:apply-templates select="." mode="SERIES"/>
                    <xsl:apply-templates select="." mode="OPTIONS"/>
                    <xsl:if test="/r:tables/@title">
                        <property name="title" type="object">
                            <property name="text" type="string"><xsl:value-of select="concat(/r:tables/@title, ' ', ./@name)" /></property>
                            <xsl:if test="$options[@key='title']">
                                <xsl:apply-templates select="$options[@key='title']/*[not(@key='text')]" />
                            </xsl:if>
                        </property>
                    </xsl:if>
                </map>
            </xsl:for-each>
        </supermap>
    </xsl:template>

    <xsl:template match="*" mode="CHART">
        <xsl:if test="$options[@key='chart']/*[(@key='inverted') or (@key='polar') or (@key='type')]">
            <xsl:message terminate="yes">Can not declare 'type', 'inverted' or 'polar' attribute of the chart's section configuration in advanced options</xsl:message>
        </xsl:if>

        <property name="chart" type="object">
            <xsl:choose>
                <xsl:when test="$axis">
                    <property name="inverted" type="boolean"><xsl:value-of select="string($axis='inverted')" /></property>
                    <property name="polar" type="boolean"><xsl:value-of select="string($axis='polar')" /></property>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:call-template name="default"> <xsl:with-param name="wanted" select="'chart'" /> </xsl:call-template>
                </xsl:otherwise>
            </xsl:choose>
            <xsl:if test="$types[@key='*']">
                <property name="type" type="string"><xsl:value-of select="$types[@key='*']/text()" /></property>
            </xsl:if>
            <xsl:apply-templates select="$options[@key='chart']/*"/>
        </property>
    </xsl:template>

    <xsl:template match="r:table" mode="AXIS">
        <xsl:if test="not($options[@key='xAxis']/*[@key='title']/*[@key='text'] and $options[@key='xAxis']/*[@key='title']/*[@key='align'] and $options[@key='xAxis']/*[@key='tickmarkPlacement']) and
                        ($options[@key='xAxis']/*[@key='title']/*[@key='text'] or $options[@key='xAxis']/*[@key='title']/*[@key='align'] or $options[@key='xAxis']/*[@key='tickmarkPlacement'])">
            <xsl:message terminate="yes">All theses advanced options for xAxis : 'title.text', 'title.align' and 'tickmarkPlacement' must be specified or none of them.</xsl:message>
        </xsl:if>
        <xsl:if test="not($options[@key='yAxis']/*[@key='labels']/*[@key='style']/*[@key='color'] and $options[@key='yAxis']/*[@key='title']/*[@key='style']/*[@key='color'] and $options[@key='yAxis']/*[@key='title']/*[@key='aling']) and
                        ($options[@key='yAxis']/*[@key='labels']/*[@key='style']/*[@key='color'] or $options[@key='yAxis']/*[@key='title']/*[@key='style']/*[@key='color'] or $options[@key='yAxis']/*[@key='title']/*[@key='aling'])">
            <xsl:message terminate="yes">All theses advanced options for yAxis : 'labels.style.color', 'title.style.color' and 'title.align' must be specified or none of them.</xsl:message>
        </xsl:if>

        <property name="xAxis" type="array">
            <object>
                <property name="categories" type="array">
                    <xsl:for-each select="./r:row">
                        <xsl:apply-templates select="$xAxis[$xAxisMode='xAxis']|ancestor-or-self::r:table/r:column_labels[$xAxisMode='unit']" mode="CATEGORIES">
                            <xsl:with-param name="row" select="." />
                        </xsl:apply-templates>
                    </xsl:for-each>
                </property>
                <xsl:choose>
                    <xsl:when test="$options[@key='xAxis']/*[@key='title']/*[@key='text'] and $options[@key='xAxis']/*[@key='title']/*[@key='align'] and $options[@key='xAxis']/*[@key='tickmarkPlacement']">
                        <xsl:apply-templates select="$options[@key='xAxis']" />
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="default">
                            <xsl:with-param name="wanted" select="'xAxis'" />
                            <xsl:with-param name="additional_object"
                                            select="$xAxis/r:column[$xAxisMode='xAxis']|ancestor-or-self::r:table/r:column_labels/r:column_label[not(@unit)][$xAxisMode='unit']" />
                        </xsl:call-template>
                        <xsl:apply-templates select="$options[@key='xAxis']" />
                    </xsl:otherwise>
                </xsl:choose>
            </object>
        </property>
        <property name="yAxis" type="array">
            <xsl:choose>
                <xsl:when test="$yAxisMode='yAxis'"><xsl:apply-templates select="$parameters" mode="YAXIS" /></xsl:when>
                <xsl:when test="$yAxisMode='unit'"><xsl:apply-templates select="." mode="YAXIS" /></xsl:when>
            </xsl:choose>
        </property>
    </xsl:template>

    <xsl:template match="r:table" mode="SERIES">
        <property name="series" type="array">
            <xsl:choose>
                <xsl:when test="$yAxisMode='yAxis'">
                    <xsl:apply-templates select="./r:column_labels/r:column_label[text()=$yAxis/*/text()]" mode="SOPTION" />
                </xsl:when>
                <xsl:otherwise>
                    <xsl:apply-templates select="./r:column_labels/r:column_label[@unit]" mode="SUNIT" />
                </xsl:otherwise>
            </xsl:choose>
        </property>
    </xsl:template>

    <xsl:template match="*" mode="OPTIONS">
        <xsl:variable name="highcharts-stacking">
            <xsl:choose>
                <xsl:when test="$stacking='none'">null</xsl:when>
                <xsl:when test="$stacking='normal'">normal</xsl:when>
                <xsl:when test="$stacking='percent'">percent</xsl:when>
                <xsl:otherwise>
                    <xsl:message terminate="yes">Unsupported chart stacking: <xsl:value-of select="$stacking"/></xsl:message>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <xsl:if test="$options[@key='plotOptions']/* or $type">
            <property name="plotOptions" type="object">
                <xsl:if test="$type">
                    <property name="{$type}" type="object">
                        <xsl:choose>
                            <xsl:when test="$highcharts-stacking='null'"><property name="stacking"><xsl:value-of select="$highcharts-stacking" /></property></xsl:when>
                            <xsl:otherwise><property name="stacking" type="string"><xsl:value-of select="$highcharts-stacking" /></property></xsl:otherwise>
                        </xsl:choose>
                        <xsl:call-template name="default"> <xsl:with-param name="wanted" select="'plotOptions'" /> </xsl:call-template>
                    </property>
                    <xsl:if test="$options[@key='plotOptions']/*[not(@key=$type)]">
                        <xsl:apply-templates select="$options[@key='plotOptions']/*[not(@key=$type)]" />
                    </xsl:if>
                </xsl:if>
                <xsl:if test="$options[@key='plotOptions']/* and not($type)">
                    <xsl:apply-templates select="$options[@key='plotOptions']/*" />
                </xsl:if>
            </property>
        </xsl:if>
        <xsl:if test="$options[not(@key='chart') and not(@key='plotOptions')
                                    and not(@key='yAxis1') and not(@key='yAxis2') and not(@key='yAxis3') and not(@key='xAxis')]">
            <xsl:apply-templates select="$options[not(@key='chart') and not(@key='plotOptions')
                                    and not(@key='yAxis1') and not(@key='yAxis2') and not(@key='yAxis3') and not(@key='xAxis')]" />
        </xsl:if>
    </xsl:template>

    <!-- xAxis.categories -->
    <xsl:template match="/r:options/r:xAxis" mode="CATEGORIES">
        <xsl:param name="row" />
        <string>
            <xsl:for-each select="$xAxis/r:column">
                <xsl:variable name="axisColumnName" select="." />
                <xsl:if test="position()>1">
                    <!-- separator between column --><xsl:value-of select="'_'" />
                </xsl:if>
                <xsl:value-of select="$row/r:column[count($row/ancestor-or-self::r:table/r:column_labels/r:column_label[text()=$axisColumnName/text()])]/text()" />
            </xsl:for-each>
        </string>
    </xsl:template>
    <xsl:template match="r:column_labels" mode="CATEGORIES">
        <xsl:param name="row" />
        <string>
            <xsl:for-each select="./r:column_label[not(@unit)]">
                <xsl:if test="position()>1">
                    <!-- separator between column --><xsl:value-of select="'_'" />
                </xsl:if>
                <xsl:value-of select="$row/r:column[count(current()/preceding-sibling::r:column_label)+1]/text()" />
            </xsl:for-each>
        </string>
    </xsl:template>
    <!-- yAxis-->
    <xsl:template match="/r:options" mode="YAXIS">
        <xsl:for-each select="$yAxis">
            <xsl:variable name="unit" select="@unit"/>
            <xsl:variable name="color">
                <xsl:choose>
                    <xsl:when test="name(.)='yAxis'"><xsl:value-of select="$highcharts-colors[1]" /></xsl:when>
                    <xsl:when test="name(.)='yAxis2'"><xsl:value-of select="$highcharts-colors[2]" /></xsl:when>
                    <xsl:when test="name(.)='yAxis3'"><xsl:value-of select="$highcharts-colors[3]" /></xsl:when>
                    <xsl:otherwise><xsl:value-of select="$highcharts-colors[1]" /></xsl:otherwise>
                </xsl:choose>
            </xsl:variable>

            <xsl:if test="./r:column">
                <object>
                    <xsl:if test="not($options[@key=name(.) or @key=concat(name(.),'1')]/*[@key='opposite'])">
                        <xsl:if test="position()>1">
                            <property name="opposite" type="boolean">true</property>
                        </xsl:if>
                    </xsl:if>
                    <xsl:choose>
                        <xsl:when test="$options[@key='yAxis1' or @key='yAxis2' or @key='yAxis3']">
                            <xsl:choose>
                                <xsl:when test="$options[@key=name(.) or @key=concat(name(.),'1')]">
                                    <xsl:if test="position()>1">
                                        <xsl:apply-templates select="$options/*[@key=name(.) or @key=concat(name(.),'1')]" />
                                    </xsl:if>
                                    <xsl:if test="not(position()>1)">
                                        <xsl:apply-templates select="$options/*[@key=name(.) or @key=concat(name(.),'1')]" />
                                    </xsl:if>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:call-template name="default">
                                        <xsl:with-param name="wanted" select="'yAxis'" />
                                        <xsl:with-param name="additional_object" select="$unit" />
                                        <xsl:with-param name="yAxisColor" select="$color" />
                                    </xsl:call-template>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:call-template name="default">
                                <xsl:with-param name="wanted" select="'yAxis'" />
                                <xsl:with-param name="additional_object" select="$unit" />
                                <xsl:with-param name="yAxisColor" select="$color" />
                            </xsl:call-template>
                        </xsl:otherwise>
                    </xsl:choose>
                    <xsl:apply-templates select="$options[@key=name(.)]|$options[@key=concat(name(.),'1')]" />
                </object>
            </xsl:if>
        </xsl:for-each>
    </xsl:template>
    <xsl:template match="r:table" mode="YAXIS">
        <xsl:variable name="highcharts-units" select="ancestor-or-self::r:table/r:column_labels/r:column_label[not(preceding-sibling::r:column_label/@unit = @unit)]/@unit" />

        <xsl:for-each select="$highcharts-units">
            <xsl:variable name="unit" select="@unit"/>
            <xsl:variable name="color" select="$highcharts-colors
[position() = count(./r:column_labels/r:column_label[@unit=$unit][1]/preceding-sibling::r:column_label[not(@unit)])+1]" />

            <object>
                <xsl:if test="position()>1">
                    <property name="opposite" type="boolean">true</property>
                </xsl:if>
                <xsl:call-template name="default">
                    <xsl:with-param name="wanted" select="'yAxis'" />
                    <xsl:with-param name="additional_object" select="$unit" />
                    <xsl:with-param name="yAxisColor" select="$color" />
                </xsl:call-template>
            </object>
        </xsl:for-each>
    </xsl:template>

    <!-- SERIES -->
    <xsl:template match="r:table/r:column_labels/r:column_label" mode="SOPTION">
        <xsl:variable name="text" select="./text()" />
        <xsl:variable name="unitNumber">
            <xsl:for-each select="$yAxis">
                <xsl:if test="./*/text() = $text">
                    <xsl:value-of select="position()"/>
                </xsl:if>
            </xsl:for-each>
        </xsl:variable>

        <xsl:if test="$unitNumber">
            <object>
                <property name="data" type="array">
                    <xsl:for-each select="ancestor-or-self::r:table/r:row/r:column[count(ancestor-or-self::r:column_label[text()=$text])+1]">
                        <number><xsl:value-of select="text()"/></number>
                    </xsl:for-each>
                </property>
                <xsl:call-template name="default">
                    <xsl:with-param name="wanted" select="'series'" />
                    <xsl:with-param name="additional_object" select="$yAxis/r:column[text()=$text]/@type" />
                    <xsl:with-param name="seriesUnitNumber" select="$unitNumber" />
                </xsl:call-template>
            </object>
        </xsl:if>
    </xsl:template>
    <xsl:template match="r:table/r:column_labels/r:column_label[@unit]" mode="SUNIT">
        <xsl:variable name="unit" select="@unit" />
        <xsl:variable name="highcharts-units"
                      select="ancestor-or-self::r:table/r:column_labels/r:column_label[@unit][not(preceding-sibling::r:column_label/@unit = @unit)]" />
        <xsl:variable name="unitNumber">
            <xsl:for-each select="$highcharts-units">
                <xsl:if test="@unit = $unit">
                    <xsl:value-of select="position()"/>
                </xsl:if>
            </xsl:for-each>
        </xsl:variable>

        <object>
            <property name="data" type="array">
                <xsl:for-each select="ancestor-or-self::r:table/r:row/r:column[count(current()/preceding-sibling::r:column_label)+1]">
                    <number><xsl:value-of select="text()"/></number>
                </xsl:for-each>
            </property>
            <xsl:call-template name="default">
                <xsl:with-param name="wanted" select="'series'" />
                <xsl:with-param name="seriesUnitNumber" select="$unitNumber" />
                <xsl:with-param name="additional_object" />
            </xsl:call-template>
        </object>
    </xsl:template>

    <!-- DEFAULT VALUES -->
    <xsl:template name="default">
        <xsl:param name="wanted" />
        <xsl:param name="additional_object" select="_" />

        <!-- Specially for yAxis -->
        <xsl:param name="yAxisColor" select="_" />
        <!-- Specially for series -->
        <xsl:param name="seriesUnitNumber" select="_" />

        <xsl:choose>
            <xsl:when test="$wanted='chart'">
                <property name="inverted" type="boolean">false</property>
                <property name="polar" type="boolean">false</property>
            </xsl:when>
            <xsl:when test="$wanted='plotOptions'">
                <property name="pointPadding" type="number">0</property>
                <property name="groupPadding" type="number">0</property>
            </xsl:when>
            <xsl:when test="$wanted='xAxis'">
                <xsl:if test="$type!='column' and $axis='polar'">
                    <property name="tickmarkPlacement" type="string">on</property>
                </xsl:if>
                <property name="title" type="object">
                    <xsl:if test="$axis!='polar'">
                        <property name="text" type="string">
                            <xsl:for-each select="$additional_object">
                                <xsl:if test="position()>1">
                                    <!-- separator between column --> <xsl:value-of select="'_'" />
                                </xsl:if>
                                <xsl:value-of select="text()" />
                            </xsl:for-each>
                        </property>
                    </xsl:if>
                    <property name="align" type="string">high</property>
                </property>
            </xsl:when>
            <xsl:when test="$wanted='yAxis'">
                <property name="labels" type="object">
                    <property name="style" type="object">
                        <property name="color" type="string"><xsl:value-of select="concat('#', $yAxisColor)" /></property>
                    </property>
                </property>
                <property name="title" type="object">
                    <property name="style" type="object">
                        <property name="color" type="string"><xsl:value-of select="concat('#', $yAxisColor)" /></property>
                    </property>
                    <property name="align" type="string">high</property>
                </property>
            </xsl:when>
            <xsl:when test="$wanted='series'">
                <property name="name" type="string"><xsl:value-of select="text()" /></property>
                <xsl:if test="$additional_object">
                    <property name="type" type="string"><xsl:value-of select="$additional_object" /></property>
                </xsl:if>
                <property name="yAxis" type="number"><xsl:value-of select="$seriesUnitNumber - 1" /></property>
            </xsl:when>
        </xsl:choose>
    </xsl:template>

    <!-- Additional 'section' and 'option' -->
    <xsl:template match="r:section">
        <property name="{@key}" type="object"> <xsl:apply-templates select="*" /> </property>
    </xsl:template>
    <xsl:template match="r:option[number(text())=text()]">
        <property name="{@name}" type="number"><xsl:value-of select="text()" /></property>
    </xsl:template>
    <xsl:template match="r:option[boolean(text())=text()]">
        <property name="{@name}" type="boolean"><xsl:value-of select="text()" /></property>
    </xsl:template>
    <xsl:template match="r:option">
        <property name="{@name}" type="string"><xsl:value-of select="text()" /></property>
    </xsl:template>
</xsl:stylesheet>
